@@ -17,10 +17,7 @@
 use super::dialect::Dialect;
 use super::sqlast::*;
 use super::sqltokenizer::*;
-use chrono::{
-    offset::{FixedOffset},
-    DateTime, NaiveDate, NaiveDateTime, NaiveTime,
-};
+use chrono::{offset::FixedOffset, DateTime, NaiveDate, NaiveDateTime, NaiveTime};
 
 #[derive(Debug, Clone)]
 pub enum ParserError {
@@ -110,6 +107,7 @@ impl Parser {
                         self.prev_token();
                         self.parse_sql_value()
                     }
+                    "CASE" => self.parse_case_expression(),
                     _ => return parser_err!(format!("No prefix parser for keyword {}", k)),
                 },
                 Token::Mult => Ok(ASTNode::SQLWildcard),
@@ -153,14 +151,14 @@ impl Parser {
                 Token::DoubleQuotedString(_) => {
                     self.prev_token();
                     self.parse_sql_value()
-                },
+                }
                 Token::LParen => {
                     let expr = self.parse();
                     if !self.consume_token(&Token::RParen)? {
                         return parser_err!(format!("expected token RParen"));
                     }
                     expr
-                },
+                }
                 _ => parser_err!(format!(
                     "Prefix parser expected a keyword but found {:?}",
                     t
@@ -196,6 +194,40 @@ impl Parser {
         }
     }
 
+    pub fn parse_case_expression(&mut self) -> Result<ASTNode, ParserError> {
+        if self.parse_keywords(vec!["WHEN"]) {
+            let mut conditions = vec![];
+            let mut results = vec![];
+            let mut else_result = None;
+            loop {
+                conditions.push(self.parse_expr(0)?);
+                self.consume_token(&Token::Keyword("THEN".to_string()))?;
+                results.push(self.parse_expr(0)?);
+                if self.parse_keywords(vec!["ELSE"]) {
+                    else_result = Some(Box::new(self.parse_expr(0)?));
+                    if self.parse_keywords(vec!["END"]) {
+                        break;
+                    } else {
+                        return parser_err!("Expecting END after a CASE..ELSE");
+                    }
+                }
+                if self.parse_keywords(vec!["END"]) {
+                    break;
+                }
+                self.consume_token(&Token::Keyword("WHEN".to_string()))?;
+            }
+            Ok(ASTNode::SQLCase {
+                conditions,
+                results,
+                else_result,
+            })
+        } else {
+            // TODO: implement "simple" case
+            // https://jakewheat.github.io/sql-overview/sql-2011-foundation-grammar.html#simple-case
+            parser_err!("Simple case not implemented")
+        }
+    }
+
     /// Parse a SQL CAST function e.g. `CAST(expr AS FLOAT)`
     pub fn parse_cast_expression(&mut self) -> Result<ASTNode, ParserError> {
         self.consume_token(&Token::LParen)?;
@@ -297,6 +329,7 @@ impl Parser {
             &Token::Mod => Ok(SQLOperator::Modulus),
             &Token::Keyword(ref k) if k == "AND" => Ok(SQLOperator::And),
             &Token::Keyword(ref k) if k == "OR" => Ok(SQLOperator::Or),
+            &Token::Keyword(ref k) if k == "LIKE" => Ok(SQLOperator::Like),
             _ => parser_err!(format!("Unsupported SQL operator {:?}", tok)),
         }
     }
@@ -318,6 +351,7 @@ impl Parser {
             &Token::Keyword(ref k) if k == "OR" => Ok(5),
             &Token::Keyword(ref k) if k == "AND" => Ok(10),
             &Token::Keyword(ref k) if k == "IS" => Ok(15),
+            &Token::Keyword(ref k) if k == "LIKE" => Ok(20),
             &Token::Eq | &Token::Lt | &Token::LtEq | &Token::Neq | &Token::Gt | &Token::GtEq => {
                 Ok(20)
             }
@@ -714,12 +748,12 @@ impl Parser {
                     },
                     Token::Number(ref n) => match n.parse::<i64>() {
                         Ok(n) => {
-//                            if let Some(Token::Minus) = self.peek_token() {
-//                                self.prev_token();
-//                                self.parse_timestamp_value()
-//                            } else {
-                                Ok(Value::Long(n))
-//                            }
+                            //                            if let Some(Token::Minus) = self.peek_token() {
+                            //                                self.prev_token();
+                            //                                self.parse_timestamp_value()
+                            //                            } else {
+                            Ok(Value::Long(n))
+                            //                            }
                         }
                         Err(e) => parser_err!(format!("Could not parse '{}' as i64: {}", n, e)),
                     },
@@ -1051,6 +1085,8 @@ impl Parser {
             None
         };
 
+        let _ = self.consume_token(&Token::SemiColon);
+
         // parse next token
         if let Some(next_token) = self.peek_token() {
             parser_err!(format!(
@@ -1107,6 +1143,8 @@ impl Parser {
             None
         };
 
+        let _ = self.consume_token(&Token::SemiColon);
+
         if let Some(next_token) = self.peek_token() {
             parser_err!(format!(
                 "Unexpected token at end of SELECT: {:?}",
@@ -1174,19 +1212,17 @@ impl Parser {
 
             // look for optional ASC / DESC specifier
             let asc = match self.peek_token() {
-                Some(Token::Keyword(k)) => {
-                    match k.to_uppercase().as_ref() {
-                        "ASC" => {
-                            self.next_token();
-                            true
-                        },
-                        "DESC" => {
-                            self.next_token();
-                            false
-                        },
-                        _ => true
+                Some(Token::Keyword(k)) => match k.to_uppercase().as_ref() {
+                    "ASC" => {
+                        self.next_token();
+                        true
                     }
-                }
+                    "DESC" => {
+                        self.next_token();
+                        false
+                    }
+                    _ => true,
+                },
                 Some(Token::Comma) => true,
                 _ => true,
             };
